public class Scacchiera {

	private boolean[][] scacchiera; // se l'elemento (i,j) è true, la corrispondente casella è occupata da una regina

	/* crea un oggetto Scacchiera di dimensione n? n vuota, cioè senza nessuna regina. */
	public Scacchiera(int n){
		scacchiera=new boolean[n][n];
		for(int i=0;i<scacchiera.length;i++)
			for(int j=0;j<scacchiera[i].length;j++)
				scacchiera[i][j]=false;
	}

	/* Posiziona una regina in posizione (i,j). Se i e j non sono indici validi la scacchiera non viene 	modificata. Se la posizione (i,j) è già occupata da una regina la scacchiera non viene modificata.  Il 	metodo restituisce true se il posizionamento è avvenuto con successo, false altrimenti */
	public boolean posizionaRegina(int i, int j){
		boolean r=false;
		int n=scacchiera.length;

		if(i>=0 && i<n && j>=0 && j<n && !scacchiera[i][j]){
			scacchiera[i][j]=true;
			r=true;
		}
		return r;
	}

	/* Restituisce il numero di regine minacciate da quella in posizione (i,j). Se in posizione (i,j) non c'è 	una regina viene restituito 0 */
	public int contaMinacciate(int i, int j){
		int count=0;
		int n=scacchiera.length;

		//controlla riga
		for(int k=0;k<n;k++){
			if(k!=j && scacchiera[i][k])
				count++;
		}

		//controlla colonna
		for(int h=0;h<n;h++){
			if(h!=i && scacchiera[h][j])
				count++;
		}

		//controlla diagonale "negativa"
		int h=i-1;
		int k=j-1;
		while(h>=0 && k>=0){
			if(scacchiera[h][k])
				count++;
			h--;
			k--;
		}

		h=i+1;
		k=j+1;
		while(h<n && k<n){
			if(scacchiera[h][k])
				count++;
			h++;
			k++;
		}

		//controlla diagonale "positiva"
		h=i-1;
		k=j+1;
		while(h>=0 && k<n){
			if(scacchiera[h][k])
				count++;
			h--;
			k++;
		}

		h=i+1;
		k=j-1;
		while(h<n && k>=0){
			if(scacchiera[h][k])
				count++;
			h++;
			k--;
		}

		return count;

	}

	/* Restituisce true se la configurazione corrente della matrice è valida (cioè ci sono n regine e 	nessuna ne minaccia un'altra), false altrimenti. */
	public boolean configurazioneValida(){
		int n=scacchiera.length;
		boolean valida=true;
		int regine=0;

		int i=0;
		while(i<n && valida){
			int j=0;
			while(j<n && valida){
				if(scacchiera[i][j]){
					regine++;
					if(contaMinacciate(i,j)!=0)
						valida=false;
				}
				j++;
			}
			i++;
		}
		valida=valida && (regine==n);

		return valida;
	}

	/* Restituisce una rappresentazione testuale della scacchiera */
	public String toString(){
		String s="";
		for(int i=0;i<scacchiera.length;i++){
			for(int j=0;j<scacchiera[i].length;j++){
				if(scacchiera[i][j]){
					s+="Q|";
				}else{
					s+="_|";
				}
			}
			s+="\n";
		}
		return s;
	}
}
